header
{
	package grammar;
	import signalgame.*;
	import signalgame.instancebased.*;
	import math.*;
	import java.util.*;
}

class P extends Parser;

	
	
game returns [StaticInformation si]
{
	si = null;
	FiniteSet 				states, messages, actions, reputations;
	ReputationPayoffMatrix 	pu, pm;
	ProbabilityDistribution Pt;
	double					correlation;
	Mutator					mutation;
	double 					mr;
	PopulationType 			pop;
	
	double [][][][][] uc, um;
	double [] pt;
	
}
		: 
		( 
		"game" 
		OPEN 
		"states" 				IS states=finiteset 		SEMICOL
		"messages" 				IS messages=finiteset 		SEMICOL
		"actions" 				IS actions=finiteset 		SEMICOL
		"reputations"			IS reputations=finiteset 	SEMICOL
		"mutator"				IS mutation=mutator			SEMICOL
		"mutationrate"			IS mr=doublev				SEMICOL
		"correlation"			IS correlation=doublev		SEMICOL
		
		("P_states"				IS pt=doublearray1	SEMICOL
			{Pt = new ProbabilityDistribution(states, pt);})
		
		("utility_communication" IS uc=doublearray5 SEMICOL 
			{pu = new ReputationPayoffMatrix(reputations,states,actions,uc);} )
			
		("utility_message" 		IS um=doublearray5 SEMICOL
			{pm = new ReputationPayoffMatrix(reputations,states,messages,um);} )
			
		"population"			pop=population
		CLOSE
		)
		{
			si = new InstanceBasedStatic(
				states, 
				messages, 
				actions,
				reputations,
				Pt,
				pu, 
				pm,
				pop,
				mutation,
				mr,
				correlation);
		}
	;


finiteset returns [FiniteSet s]
{
	s = null;
	Vector v = new Vector();
	String i;
}
		:	OPEN (i=identifier (COMMA)? {v.add(i);} )* CLOSE
			{ s = new FiniteSet(v.toArray()); }
		;


doublearray1 	returns [double [] x]
{
	x = null;
	double d;
	Vector v = new Vector();
}
		:	OPEN (d=doublev (COMMA)? {v.add(new Double(d)); })* CLOSE
			{
				x = new double[v.size()];
				for (int i = 0; i < x.length; i++)
				{
					x[i] = ((Double)v.elementAt(i)).doubleValue();
				}
			}
		;
		

doublearray2	returns [double [][] x]
{
	x = null;
	Vector v = new Vector();
	double [] a;
}
		:	OPEN (a=doublearray1 (COMMA)? {v.add(a);} )* CLOSE
			{
				x = new double[v.size()][];
				for (int i = 0; i < x.length; i++)
				{
					x[i] = (double [])v.elementAt(i);
				}
			}
		;

doublearray3	returns [double [][][] x]
{
	x = null;
	Vector v = new Vector();
	double [][] a;
}
		:	OPEN (a=doublearray2 (COMMA)? {v.add(a);} )* CLOSE
			{
				x = new double[v.size()][][];
				for (int i = 0; i < x.length; i++)
				{
					x[i] = (double [][])v.elementAt(i);
				}
			}
		;

doublearray4	returns [double [][][][] x]
{
	x = null;
	Vector v = new Vector();
	double [][][] a;
}
		:	OPEN (a=doublearray3 (COMMA)? {v.add(a);} )* CLOSE
			{
				x = new double[v.size()][][][];
				for (int i = 0; i < x.length; i++)
				{
					x[i] = (double [][][])v.elementAt(i);
				}
			}
		;



doublearray5	returns [double [][][][][] x]
{
	x = null;
	Vector v = new Vector();
	double [][][][] a;
}
		:	OPEN (a=doublearray4 (COMMA)? {v.add(a);} )* CLOSE
			{
				x = new double[v.size()][][][][];
				for (int i = 0; i < x.length; i++)
				{
					x[i] = (double [][][][])v.elementAt(i);
				}
			}
		;
		
		
population	returns [ PopulationType pt ] 
{
	pt = null;
	Vector v = new Vector();
	IndividualType it;
}
		:
			OPEN
			(it=individualtype {v.add(it);} )*
			CLOSE
			{
				IndividualType [] is = new IndividualType[v.size()];
				for (int i = 0; i < is.length; i++)
				{
					is[i] = (IndividualType)v.elementAt(i);
				}
				
				pt = new PopulationType(is);
			}
		;


individualtype returns [IndividualType it ] 
{
	it = 						null;
	int             			count;
	boolean         			b;
	ReputationStrategyType    	Ss, Sh;
	String          			reputation;
}
	:	"individual" OPEN count=intv COMMA b=evolution CLOSE	
		OPEN
			"speakerstrategy" 	Ss=reputationstrategy
			"hearerstrategy" 	Sh=reputationstrategy
			"reputation" 		OPEN reputation=identifier CLOSE SEMICOL
		CLOSE
		{
			it = new IndividualType(count, b, Ss, Sh, reputation);
		}
		;

evolution returns [boolean b]
{
	b = false;
}
		:	"evolving" 	{ b = true; }
		| 	"fixed"		{ b = false; }
		;
		

reputationstrategy returns [ ReputationStrategyType rst ]
{
	rst = null;
	Vector v = new Vector();
	StrategyType s;
	
}
	:	OPEN ( s=strategy (COMMA)? {v.add(s);} )*  CLOSE
		{
			StrategyType [] st = new StrategyType [v.size()];

			for (int i = 0; i < st.length; i++)
			{
				st[i] = (StrategyType)v.elementAt(i);
			}
			
			rst = new ReputationStrategyType(st);
		}
	;
		
		


strategy returns [ StrategyType st ]
{
	st = null;
	Vector mv = new Vector();
	MappingType m;
}
	:	OPEN
			(m=mappingtype {mv.add(m); })*
		CLOSE
			{
				MappingType [] ms = new MappingType[mv.size()];
				for (int i = 0; i < ms.length; i++)
				{
					ms[i] = (MappingType)mv.elementAt(i);
				}
				
				st = new StrategyType(ms);
			}
	;
	
mappingtype returns [MappingType m]
{
	PDFactory 	p = null;
	boolean 	b = false;
	m = null;
	
}
	: "map" OPEN p=pdf COMMA b=evolution CLOSE SEMICOL 
		{ m = new MappingType(p,b); }
	;	
	

function :
	ID OPEN (ID (COMMA)?)* CLOSE
	;	
	
	
pdf returns [PDFactory pdf]
{
	pdf = null;
	double [] ps;
}
		: "random" 			{pdf = new RandomPDF(); }
		| "justone"			{pdf = new JustOnePDF();}
		| "uniform"			{pdf = new UniformPDF();}
		| ps=doublearray1	{pdf = new GivenPDF(ps);}
	;
	
mutator returns [Mutator mutator]
{
	mutator = null;
	double v;
}
		: "identity" OPEN CLOSE					{mutator = new IdentityMutator(); }
		| "fairshare" OPEN v=doublev CLOSE		{mutator = new FairShareMutator(v);}
		| "unfairshare" OPEN v=doublev CLOSE	{mutator = new UnfairShareMutator(v);}
		| "exchange" OPEN CLOSE					{mutator = new ExchangeMutator();}
	;
	
	
identifier returns [String s]
{
	s = null;
}	
	: i:ID 
		{ s = i.getText(); }
	;

intv returns [int i]
{
	i = 0;
	String s = "";
}
		: 	(x:DIGIT {s+=x.getText();} )+
			{ i = Integer.parseInt(s); }
		;
		
doublev returns [double d]
{
	d = 	0;
	String s = "";
}
		:	(
				x:DIGIT 
				{s+=x.getText();}
			)+ 
			(
				y:PERIOD 
				{s+=y.getText();} 
				(
					z:DIGIT 
					{s+=z.getText();} 
				)+
			)?
			{ d = Double.parseDouble(s); }
		;
		



	
class L extends Lexer;

DIGIT	: ('0'..'9')
		;

protected 
LETTER	:	('a'..'z' | 'A'..'Z')
		;
		
protected
UNDERSCORE :	'_'
		;		
		
ID		:	LETTER (LETTER | DIGIT | UNDERSCORE)*
		;		
		
OPEN	:	( '{' | '(' )
		;
		
CLOSE	:	( '}' | ')' )
		;
		
IS		:	'='
		;
		
MINUS	:	'-'
		;		
		
		
SEMICOL	:	';'
		;

COMMA	:	','	
		;

PERIOD 	:	'.'
		;		


// whitespace
WS		:	(	' '
			|	'\t'
			|	'\r' '\n' { newline(); }
			|	'\n' { newline(); }
			)
			{$setType(Token.SKIP);}	//ignore this token
		;

