package gui;

import grammar.*;
import java.io.*;
import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.event.*;
import javax.swing.filechooser.*;
import signalgame.*;
import signalgame.typebased.*;


/**
 *  @author Project A.I. Development Team
 */

public class GUI extends JFrame
{

    BorderLayout borderLayout5 = new BorderLayout();
    JToolBar jToolBar1 = new JToolBar();
    JButton bStep100 = new JButton();
    JButton bStep10 = new JButton();
    JButton bStep1 = new JButton();
    JScrollPane jScrollPane1 = new JScrollPane();

	GUI_Evolution ge = new GUI_Evolution();


	GUI_Settings gs = new GUI_Settings(this);


	StaticInformation   si =        null;
	DynamicInformation  di =        null;
	String              ni =        null;

    GUI_Info            pInfo =   new GUI_Info();
    JButton bStep1000 = new JButton();
    JButton bStop = new JButton();

	Calculator  calculator = null;
    JMenuBar jMenuBar1 = new JMenuBar();
    JMenu mSimulation = new JMenu();
    JMenu mNew = new JMenu();
    JMenuItem mEnd = new JMenuItem();
    JMenu mHelp = new JMenu();
    JMenuItem mAbout = new JMenuItem();


	GameAcquirer [] ga = {new TypeBased(), new InstanceBased()};
    JMenuItem mRestart = new JMenuItem();

	GUI This = this;
    JPanel jPanel2 = new JPanel();
    BorderLayout borderLayout1 = new BorderLayout();


    public GUI()
    {
        try
        {
			this.setSize(800,600);
            jbInit();
			this.setVisible(true);
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }

    private void jbInit() throws Exception
    {
        this.setDefaultCloseOperation(3);
        this.setJMenuBar(jMenuBar1);
        this.getContentPane().setLayout(borderLayout5);
        bStep100.setText("100 Steps");
        bStep100.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                startCalculation(100);
            }
        });
        bStep10.setText("10 Steps");
        bStep10.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                startCalculation(10);
            }
        });
        bStep1.setText("One Step");
        bStep1.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                startCalculation(1);
            }
        });
        bStep1000.setText("1000 Steps");
        bStep1000.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                startCalculation(1000);
            }
        });
        bStop.setEnabled(false);
        bStop.setText("Stop");
        bStop.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                stopCalculation();
            }
        });
        mSimulation.setText("Simulation");
        mNew.setText("New");
        mEnd.setText("End");
        mEnd.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                End(e);
            }
        });
        mHelp.setText("Help");
        mAbout.setText("About");
        mRestart.setText("Restart");
        mRestart.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                Restart(e);
            }
        });
        jPanel2.setLayout(borderLayout1);
        jPanel2.setBorder(BorderFactory.createLoweredBevelBorder());
        this.getContentPane().add(jToolBar1,  BorderLayout.SOUTH);
        jToolBar1.add(bStep1, null);
        jToolBar1.add(bStep10, null);
        jToolBar1.add(bStep100, null);
        jToolBar1.add(bStep1000, null);
        jToolBar1.add(bStop, null);
        this.getContentPane().add(jPanel2, BorderLayout.CENTER);
        jPanel2.add(pInfo,  BorderLayout.NORTH);
        jPanel2.add(jScrollPane1, BorderLayout.CENTER);
        jScrollPane1.getViewport().setView(ge);
        jMenuBar1.add(mSimulation);
        jMenuBar1.add(mHelp);
        mSimulation.add(mNew);
        mSimulation.add(mRestart);
        mSimulation.add(mEnd);
        mHelp.add(mAbout);


		for (int i = 0; i < ga.length; i++)
		{
			mNew.add(ga[i]);
		}

		ge.setSelectionListener(pInfo);


		setGame(null, null);
    }


	public static void main(String [] args)
	{
		new GUI();
	}


	public void setInterface(boolean stepButtons, boolean menuButtons, boolean stopButton, boolean simControl)
	{
		bStep1.setEnabled(stepButtons);
		bStep10.setEnabled(stepButtons);
		bStep100.setEnabled(stepButtons);
		bStep1000.setEnabled(stepButtons);
		bStop.setEnabled(stopButton);
		mSimulation.setEnabled(menuButtons);
		mHelp.setEnabled(menuButtons);
		mRestart.setEnabled(simControl);
		mEnd.setEnabled(simControl);
	}

    synchronized void startCalculation(int steps)
    {
		calculator = new Calculator(steps);
		calculator.start();
    }

    synchronized void stopCalculation()
    {
//		try
//		{
			calculator.kill();
//			calculator.join();
			calculator = null;
//		}
//		catch (InterruptedException ex)
//		{
//			throw new Error(ex);
//		}
    }


	public void setGame(StaticInformation si, String name)
	{
		// Clear Evolution Panel
		ge.clear();
		pInfo.reset();

		String title = "Signal Game";


		if (si == null)
		{
			this.si = null;
			this.di = null;
			this.ni =  null;
			title += "(No Game)";
		}
		else
		{
			this.si = si;
			this.di = si.createDyamic();
			this.ni =  name;
			title +=  " - " + ni;
			ge.addStep(di);
		}

		this.setInterface(si != null, true, false, si != null);
		this.ge.setEnabled(si != null);

		super.setTitle(title);
	}


    void New(ActionEvent e)
    {
		TypeBasedStatic sg = gs.getTypeBasedStatic();
		if (sg != null) setGame(sg, "GUI Specified");
    }


    void End(ActionEvent e)
    {
		setGame(null, null);
    }


	class Calculator extends Thread
	{
		boolean running;
		int     togo;

		public Calculator(int togo)
		{
			this.togo = togo;
		}

		public void run()
		{
			setInterface(false, false, true, false);

			for (running = true; running && togo > 0; togo--)
			{
				di = di.evolve();
				ge.addStep(di);
			}

			setInterface(true, true, false, true);
		}

		public void kill()
		{
			running = false;
		}
	}


	abstract class GameAcquirer extends JMenuItem implements ActionListener
	{
		public GameAcquirer(String name)
		{
			super(name + "...");
			this.addActionListener(this);
		}

	}

	class TypeBased extends GameAcquirer
	{
		public TypeBased()
		{
			super("Type Based");
		}

		public void actionPerformed(ActionEvent e)
		{
			TypeBasedStatic sg = gs.getTypeBasedStatic();
			if (sg != null) setGame(sg, "GUI Specified");
		}
	}

	class InstanceBased extends GameAcquirer
	{
		public InstanceBased()
		{
			super("Instance Based");
		}

		public void actionPerformed(ActionEvent e)
		{
			JFileChooser chooser = new JFileChooser(".");

			int returnVal = chooser.showOpenDialog(This);
			if(returnVal == JFileChooser.APPROVE_OPTION)
			{
				try
				{
					File f = chooser.getSelectedFile();
				    L lexer = new L(new DataInputStream(new FileInputStream(f)));
				    P parser = new P(lexer);
				    StaticInformation sg = parser.game();
					setGame(sg, f.getCanonicalFile().getName());
				}
				catch (Exception ex)
				{
					ex.printStackTrace();
					JOptionPane.showMessageDialog(This, ex, "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}

    void Restart(ActionEvent e)
    {
		this.setGame(si, ni);
    }

}
