package gui;

import java.util.Vector;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import signalgame.*;
import signalgame.typebased.*;


/**
 *  @author Project A.I. Development Team
 */

public class GUI_Evolution extends JPanel implements MouseMotionListener, MouseListener
{
	Vector sgis;

	public static final int HEIGHT =    20;
	public static final int SPACE =      4;
	public static final int MARGIN =    60;

	GUI_Info    selectionListener = null;

	int     selectedx = -1;
	int     selectedy = -1;


    public GUI_Evolution()
    {
		sgis = new Vector();
        try
        {
            jbInit();
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }

    }

    private void jbInit() throws Exception
    {
        setEnabled(this.isEnabled());
		this.addMouseMotionListener(this);
		this.addMouseListener(this);
    }

	public void setSelectionListener(GUI_Info listener)
	{
		selectionListener = listener;
	}


	public void mouseDragged(MouseEvent e)
	{
		mouseMoved(e);
	}

	public void mouseMoved(MouseEvent e)
	{
		this.selectedx = e.getX();
		this.selectedy = e.getY();
		updateSelection();
	}

	public void updateSelection()
	{
		int row = selectedy/HEIGHT;

		if (row >= sgis.size() || selectedx < MARGIN)
		{
			selectionListener.setSettings(null, 0);
		    return;
		}

		DynamicInformation di = (DynamicInformation)sgis.elementAt(sgis.size()-1 - row);

		double p = (double)(selectedx-MARGIN)/(getSize().getWidth() - MARGIN);


		double cumul = 0;

		for (int j = 0; j < di.I.length; j++)
		{
			double pj = di.Pi.ps[j];

			if (p < cumul + pj)
			{
				selectionListener.setSettings(di, j);
				return;
			}
			cumul += pj;
		}

		selectionListener.setSettings(null, 0);

	}

	public void mousePressed(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) { selectedx = selectedy = -1; updateSelection(); }



	public void clear()
	{
		sgis.clear();
	}

	public void setEnabled(boolean b)
	{
		super.setEnabled(b);
		if (b)  this.setBackground(Color.black);
		else    this.setBackground(Color.lightGray);
	}


	public void addStep(DynamicInformation sgi)
	{
		sgis.add(sgi);
		this.setPreferredSize(new Dimension(10, sgis.size() * HEIGHT));
		this.updateSelection();
		this.revalidate();
		this.repaint();
	}


	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);

		double width = this.getSize().getWidth() - MARGIN;

		Rectangle cb = g.getClipBounds();
		// Calculate rows in normal coordinate system
		int scStartRow = (cb.y)/HEIGHT;
		int scEndRow =   (cb.y + cb.height)/HEIGHT;

		// Calculate rows in bottom-up coordinate system, and crop to sgis
		int startRow =  Math.max(0,                 sgis.size() - 1 - scEndRow);
		int endRow =    Math.min(sgis.size() -1,    sgis.size() - 1 - scStartRow);

		// Iterate over all time steps that need to be redrawn
		for (int i = startRow; i <= endRow; i++)
		{
			DynamicInformation di = (DynamicInformation)sgis.elementAt(i);

			double  cumul = 0;
			int y = (sgis.size()-1 - i )*HEIGHT;

			g.setColor(Color.white);
			g.drawString(i+":", 2, y+HEIGHT-SPACE);

			// Calculate maximum and minimum expected value
			double min = Double.MAX_VALUE;
			double max = Double.MIN_VALUE;

			for (int j = 0; j < di.Ei.length; j++)
			{
				min = Math.min(min, di.Ei[j]);
				max = Math.max(max, di.Ei[j]);

			}

			// Calculate the range in which the values lie (corrected for empty range)
			double range = max-min;
			if (range < 1.0E-5) { min = min-1; max = max+1; range = 2;};

			if (di.E-min < max-di.E)  min = 2*di.E - max;
			else                      max = 2*di.E - min;
		    range = max-min;

			// Iterate over all individuals
			for (int j = 0; j < di.I.length; j++)
			{
				double pj = di.Pi.ps[j];

				int x = MARGIN+(int)(cumul * width);
				int w = (int)(pj * width);

				// Draw a box
				g.setColor(Color.white);
				g.drawRect(x, y, w, HEIGHT-SPACE);

				// Filled with the color
				g.setColor(GUI_Legend.heat((di.Ei[j]-min)/range));
				g.fillRect(x+1, y+1, w-2, HEIGHT-SPACE-2);

				cumul += pj;
			}
		}
	}
}