package gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import math.*;

/**
 *  @author Project A.I. Development Team
 */


interface ProbabilityFactory
{
	public ProbabilityDistribution create(FiniteSet set);
}

class Uniform implements ProbabilityFactory
{
	public ProbabilityDistribution create(FiniteSet set)
	{
		return ProbabilityDistribution.uniform(set);
	}

	public String toString() { return "Uniform"; }
}

class Random implements ProbabilityFactory
{
	public ProbabilityDistribution create(FiniteSet set)
	{
		return ProbabilityDistribution.random(set);
	}

	public String toString() { return "Random"; }
}

class JustOne implements ProbabilityFactory
{
	public ProbabilityDistribution create(FiniteSet set)
	{
		return ProbabilityDistribution.justOne(set);
	}

	public String toString() { return "Just One"; }
}

public class GUI_Probability extends JPanel
{
	public static final ProbabilityFactory [] factories = {new Uniform(), new Random(), new JustOne()};

    BorderLayout borderLayout1 =        new BorderLayout();
    GUI_ProbabilityDisplay jPanel1 =    new GUI_ProbabilityDisplay();
    JComboBox cbType =                  new JComboBox(factories);

	FiniteSet                       s;
	ProbabilityDistribution         p;

    public GUI_Probability()
    {
        try
        {
            jbInit();
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }


	private void jbInit() throws Exception
    {
        this.setLayout(borderLayout1);
        cbType.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                setType();
            }
        });
        this.add(jPanel1, BorderLayout.CENTER);
        this.add(cbType, BorderLayout.SOUTH);

		this.cbType.setSelectedIndex(0);
    }

	public void setSets(FiniteSet set)
	{
		s = set;
		setType();
	}

	public ProbabilityDistribution getProbabilityDistribution()
	{
		assert(p != null);
		return p;
	}

    void setType()
    {
		if (s == null) return;

		p = ((ProbabilityFactory)this.cbType.getSelectedItem()).create(s);

		jPanel1.setProbability(p);
    }
}