package gui;

import javax.swing.*;
import java.awt.*;
import math.*;

/**
 *  @author Project A.I. Development Team
 */

public class GUI_ProbabilityDisplay extends JPanel
{
	ProbabilityDistribution pd;

	Rectangle xlabel;
	Rectangle ylabel;

	static final int MARGIN = 20;
	static final int SPACE = 5;
	static final int YLABELS = 4;

    public GUI_ProbabilityDisplay()
    {
        try
        {
            jbInit();
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }

    private void jbInit() throws Exception
    {
        this.setBackground(Color.white);
		this.setPreferredSize(new Dimension(500,150));
    }

	public void setProbability(ProbabilityDistribution pd)
	{
	    xlabel = new Rectangle();
		ylabel = new Rectangle();

		if (pd == null)
		{
			this.setEnabled(false);
		}
		else
		{
			this.setEnabled(true);

			Graphics g = this.getGraphics();
			FontMetrics f = g.getFontMetrics();

			for (int i = 0; i <=YLABELS; i++)
			{
				xlabel = xlabel.union(f.getStringBounds(Double.toString((double)i/YLABELS), g).getBounds());
			}

			for (int i = 0; i < pd.domain.elements.length; i++)
			{
				ylabel = ylabel.union(f.getStringBounds(Integer.toString(i), g).getBounds());
			}
		}
		this.pd = pd;

		this.revalidate();
		this.repaint();
	}


	public void setEnabled(boolean b)
	{
		super.setEnabled(b);
		if (b)  this.setBackground(Color.white);
		else    this.setBackground(Color.lightGray);
	}


	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);

		if (pd == null) return;

		Dimension size = this.getSize();

		Rectangle frame = new Rectangle(xlabel.width + 2*SPACE,                       size.height - ylabel.height - 2*SPACE,
									    size.width - xlabel.width - 2*SPACE - MARGIN, -size.height + ylabel.height + 2*SPACE + MARGIN);

		g.setColor(Color.black);
		g.drawLine(frame.x, frame.y, frame.x + frame.width, frame.y);
		g.drawLine(frame.x, frame.y, frame.x, frame.y + frame.height);


		for (int i = 0; i <=YLABELS; i++)
		{
			g.setColor(Color.black);

			int y = frame.y + frame.height*i/YLABELS;
			g.drawLine( frame.x - 2, y, frame.x + 2, y);

			g.drawString(Double.toString((double)i/YLABELS), -xlabel.x + SPACE, y - xlabel.y/2);
		}

		Dimension d = this.getSize();
		int labels = d.width/ylabel.width/2;
		int labelstep = Math.max(1, pd.domain.elements.length / labels);


		for (int i = 0; i < pd.domain.elements.length; i+= labelstep)
		{
			int x = frame.x + (int)((i+0.5) * frame.width / pd.domain.elements.length);

		    g.setColor(Color.black);
			g.drawString(Integer.toString(i), x - ylabel.x, frame.y - ylabel.y + SPACE);
		}

		for (int i = 0; i < pd.domain.elements.length; i++)
		{
			int x = frame.x + (int)((i+0.5) * frame.width / pd.domain.elements.length);

			g.setColor(Color.blue);
			g.drawLine( x, frame.y,
						x, frame.y + (int)(frame.height * pd.ps[i]));
		}

	}
}