package math;

import java.util.*;


/** Models a finite set of basic elements.
 *  @author Project A.I. Development Team
 */


public class FiniteSet
{
	public final Object []  elements;
	public final boolean    multiset;

	/** Constructs a new finite (semi-ordered) set.
	 *  The constructor verifies that every element occurs at most once.
	 *
	 * @param elements The elements to be contained in the set
	 */

	 public FiniteSet(Object [] elements)
	 {
		this(elements, false);
	 }

	public FiniteSet(Object [] elements, boolean multiset)
    {
		this.elements = elements;
		this.multiset = multiset;

		if (!multiset)
		{
			// Check that we truly are a set
			for (int i = 0; i < elements.length; i++)
			{
				for (int j = i+1; j < elements.length; j++)
				{
					assert !elements[i].equals(elements[j]) : "Duplicate element: " + elements[i];
				}
			}
		}
    }

	public int getElementIndex(Object o)
	{
		for (int i = 0; i < elements.length; i++)
		{
			if (o.equals(elements[i])) return i;
		}

		throw new Error("Element " + o + " not in " + this);
	}

	/** Computs Cartesian Product of two sets
	 *
	 * @param s1 The first set of type S1
	 * @param s2 The second set of type S2
	 * @return A set of type S1 x S2 (thus containing tuples)
	 */

	public static FiniteSet cartesianProduct(FiniteSet s1, FiniteSet s2)
	{
		Object [] result = new Object[s1.elements.length * s2.elements.length];

		int index = 0;
		for (int i1 = 0; i1 < s1.elements.length; i1++)
		{
			for (int i2 = 0; i2 < s2.elements.length; i2++)
			{
				result[index++] = new Tuple(new Object[] {s1.elements[i1], s2.elements[i2]});
			}
		}

		return new FiniteSet(result);
	}

	/** Generates a string representation of this set on multiple lines
	 *
	 * @return the string
	 */

	public String multiline()
	{
		StringBuffer s = new StringBuffer();
		boolean first = true;

		s.append("{ ");

		for (int i = 0; i < elements.length; i++)
		{
			if (!first) s.append(",\n  ");
			else        first = false;

			s.append(elements[i]);
		}
		s.append("}");

		return s.toString();
	}


	/** Generates a string representation of this set on one line
	 *
	 * @return the string
	 */

	public String toString()
	{
		StringBuffer s = new StringBuffer();
		boolean first = true;

		s.append('{');

		for (int i = 0; i < elements.length; i++)
		{
			if (!first) s.append(", ");
			else        first = false;

			s.append(elements[i]);
		}
		s.append('}');

		return s.toString();
	}
}
