package math;

import util.*;


/** Models a probabilistic function from 'domain' to 'range'.
 *  @author Project A.I. Development Team
 */


public class ProbabilisticFunction implements Function
{
	public static final int LABELW = 25;

	public final FiniteSet                      domain;
	public final FiniteSet                      range;
	public final ProbabilityDistribution []     ps;

    public ProbabilisticFunction(FiniteSet domain, ProbabilityDistribution [] ps)
    {
		assert domain.elements.length == ps.length : "Domain size is " + domain.elements.length + " while mapping is of size " + ps.length;
		assert domain.elements.length >= 1  : "There must be at least one element to map from";
		this.domain = domain;
		this.range = ps[0].domain;

		for (int i = 0; i < ps.length; i++)
		{
			assert ps[i].domain.equals(range) : "Function range inconsistent";
		}

		this.ps = ps;
    }

	public ProbabilisticFunction(DiscreteFunction s)
	{
		this.domain =   s.domain;
		this.range =    s.range;
		this.ps =       new ProbabilityDistribution[domain.elements.length];

		for (int i = 0; i < ps.length; i++)
		{
			double [] ps2 = new double [range.elements.length];
			ps2[s.mapping[i]] = 1.0;
			ps[i] = new ProbabilityDistribution(range, ps2);
		}
	}


	public double distance(Function F)
	{
		ProbabilisticFunction pf = (ProbabilisticFunction)F;

		double d2 = 0;


		for (int i = 0; i < ps.length; i++)
		{
			double d = ps[i].distance(pf.ps[i]);
			d2 += d*d;
		}

		return Math.sqrt(d2);
	}




	public String multiline()
	{
		StringBuffer s = new StringBuffer();

		s.append(Formatter.padr("", LABELW));
		for (int j = 0; j < range.elements.length; j++)
		{
			s.append(Formatter.padr(range.elements[j], LABELW));
		}

		for (int i = 0; i < domain.elements.length; i++)
		{
			s.append('\n');
			s.append(Formatter.padr(domain.elements[i], LABELW));
			for (int j = 0; j < range.elements.length; j++)
			{
				s.append(Formatter.padr(ps[i].ps[j], LABELW));
			}
		}

		return s.toString();
	}


	public String toString()
	{
		return this.multiline();
	}

}