package signalgame;

import java.util.HashMap;

import math.*;
import util.*;


public class ReputationPayoffMatrix
{
	public final FiniteSet 			set4;
	public final PayoffMatrix [][] 	matrices;


    public ReputationPayoffMatrix(FiniteSet set4, FiniteSet set1, FiniteSet set2, double [][][][][] values)
    {
		matrices = new PayoffMatrix[set4.elements.length][set4.elements.length];
		// Check the values structure for consistency with row and col labels
		assert values.length == set4.elements.length : "1st dimension of Values (" + values.length + ") should be as large as |set4| (" + set4.elements.length +")";

		for (int si = 0; si < set4.elements.length; si++)
		{
			assert values[si].length == set4.elements.length : "2nd dimension of Values (" + values[si].length + ") should be as large as |set4| (" + set4.elements.length +")";

			for (int ai = 0; ai < set4.elements.length; ai++)
			{
				matrices[si][ai] = new PayoffMatrix(set1, set2, values[si][ai]);
			}
		}

		this.set4 =     set4;
    }

	/** Generates a nice string representation of this funciton on multiple lines.
	 *  The string layout is computed in such a way that it always fits.
	 *  The column headers are centered above the columns
	 *
	 * @return The string
	 */


	public String multiline()
	{
		StringBuffer s = new StringBuffer();

		for (int r = 0; r < set4.elements.length; r++)
		{
			for (int c = 0; c < set4.elements.length; c++)
			{
				s.append("Reputation(" + set4.elements[r] + "," + set4.elements[c] + "):\n");
				s.append(matrices[r][c].multiline());
				s.append('\n');
			}
		}

		return s.toString();
	}

	/** Generates a string representation of this funciton
	 *  This function just calls 'multiline', because a one-dimensional representation
	 *  of a Matrix is neither readable nor useful.
	 *
	 * @return The string
	 */


	public String toString()
	{
		return multiline();
	}
}
