package signalgame.instancebased;

import javax.swing.*;
import java.awt.*;

import math.*;
import signalgame.*;
import signalgame.ReputationPayoffMatrix;

/** The static information in a population based Signal Game.
 *  @author Project A.I. Development Team
 */


 // Indivudials are <Speaker strategy, Hearer strategy, Reputation>
 //
 // Ss is a probabilistic speaker strategy (State ->    Probability over Messages)
 // Hs is a probabilistic hearer strategy  (Message ->  Probability over Actions)

 // So Utility payoffs for State x SType x Hs x HType
 // and Costs defined for State x SType x Message and



public class InstanceBasedStatic extends StaticInformation
{
	public final    PopulationType              pt;
	public final    TheGoodMutate               Es = new TheGoodMutate();
	public final    double                      mutationrate;
	public final    double                      correlation;

    public InstanceBasedStatic(  FiniteSet T, FiniteSet M, FiniteSet A, FiniteSet R,
						ProbabilityDistribution Pt,
						ReputationPayoffMatrix Utilities, ReputationPayoffMatrix Costs,
						Mutator mutator)
    {
		this(T, M, A, R, Pt, Utilities, Costs, null, mutator, 0, 0);
    }

    public InstanceBasedStatic(  FiniteSet T, FiniteSet M, FiniteSet A, FiniteSet R,
						ProbabilityDistribution Pt,
						ReputationPayoffMatrix Utilities, ReputationPayoffMatrix Costs,
						PopulationType pt,
						Mutator mutator, double mutationrate, double correlation)
    {
		super(T, M, A, R, Pt, Utilities, Costs, mutator);
		this.pt =           pt;
		this.mutationrate = mutationrate; assert mutationrate >= 0.0 && mutationrate <= 1.0 : mutationrate + " is not a rate (sould be in [0,1])";
		this.correlation =  correlation;
    }

	public String multiline()
	{
		StringBuffer s = new StringBuffer("Signal Game Configuration (Instance Based):");

		s.append("\nState Set:          T = " + T);
		s.append("\nAction Set:         A = " + A);
		s.append("\nMessage Set:        M = " + M);
		s.append("\nReputation Set:     R = " + R);
		s.append("\nProbabilities:      Pt= " + Pt);
		s.append("\nUtility Matrix:     UM = " + Utilities.multiline());
		s.append("\nCost Matrix:        CM = " + Costs.multiline());

		return s.toString();
	}


	public DynamicInformation createDyamic()
	{
		Individual [] I =pt.createPopulation(this);
		ProbabilityDistribution Pi = ProbabilityDistribution.uniform(new FiniteSet(I));
		return new InstanceBasedDynamic(this, I, Pi);
	}
}