package signalgame.instancebased;

import java.util.*;
import math.*;
import signalgame.*;

/**
 */


public class TheGoodMutate
{
	public static final double DROP_N_MUTATE = 0.3;

	public DynamicInformation update(DynamicInformation di)
	{
		 final InstanceBasedDynamic pi = (InstanceBasedDynamic)di;

		// Randomize the population to overcome stable sorting
		Integer [] is = permutation(pi.I.length);

		// Sort the population indices to increasing utility
		Arrays.sort(is, new Comparator()
			{
				public int compare(Object o1, Object o2)
				{
					assert(o1 instanceof Integer && o2 instanceof Integer);
					int i1 = ((Integer)o1).intValue();
					int i2 = ((Integer)o2).intValue();

					// Compare reversed, to sort hight to low
					return Double.compare(pi.Ei[i2], pi.Ei[i1]);
				}
			}
		);

		// Create the new population
		IIndividual [] npi = new Individual [pi.I.length];


		int firstMutate = (int)((1.0-DROP_N_MUTATE)*pi.I.length);

		// Copy the best individuals to the first locations
		for (int i = 0; i < firstMutate; i++)
		{
			npi[i] = di.I[is[i].intValue()];
		}

		// Then fill the rest with mutations of the good individuals
		for (int i = firstMutate; i < npi.length; i++)
		{
			npi[i] = mutate((Individual)di.I[is[i-firstMutate].intValue()], pi.S.mutator);
		}

		return new InstanceBasedDynamic(pi.S, npi, pi.Pi);
	}

	public Integer [] permutation(int n)
	{
		Integer [] is = new Integer[n];

		// Generate a random permutation (to overcome stable sorting)
		for (int i = 0; i < n; i++)
		{
			int j = (int)(Math.random()*(i+1));
			is[i]=is[j];
			is[j]=new Integer(i);
		}

		return is;
	}

	public Individual mutate(Individual t, Mutator mutator)
	{
		return new Individual(t.type,   mutate(t.Ss, t.type.Ss, mutator),
									    mutate(t.Sh, t.type.Sh, mutator), t.reputationID);
	}

	public ReputationFunction mutate(ReputationFunction r, ReputationStrategyType t, Mutator mutator)
	{
		Function [] f = new Function[r.fs.length];

		for (int i = 0; i < f.length; i++)
		{
			f[i] = mutate((ProbabilisticFunction)r.fs[i], t.st[i], mutator);
		}
		return new ReputationFunction(r.domain, f);
	}


	public ProbabilisticFunction mutate(ProbabilisticFunction s, StrategyType t, Mutator mutator)
	{
		ProbabilityDistribution [] ps = new ProbabilityDistribution[s.ps.length];

		for (int i = 0; i < ps.length; i++)
		{
			ps[i] = mutate(s.ps[i], t.ms[i], mutator);
		}

		return new ProbabilisticFunction(s.domain, ps);
	}

	public ProbabilityDistribution mutate(ProbabilityDistribution p, MappingType t, Mutator mutator)
	{
		if (!t.evolving) return p;

		return mutator.mutate(p);
	}

}