package signalgame.typebased;

import math.*;
import util.*;
import signalgame.*;



/** The dynamic information in the Probability based Signal Game.
 *  @author Project A.I. Development Team
 */

public class TypeBasedDynamic extends DynamicInformation
{
	public static final boolean DEBUG = false;
//	public final    TypeBasedStatic             sg;
//  public final	ProbabilityDistribution     Pi;
//	public final    double []                   Ei;
//	public final    double                      E;

	/** Constructs a TypeBasedInstance object, containing the dynamic information
	 *  during simulation of the signal game
	 *
	 * @param sg    The Signal Game this object refers to
	 * @param Pi    The probability distribution over individuals
	 */

    public TypeBasedDynamic(StaticInformation sg, IIndividual [] I, ProbabilityDistribution Pi)
    {
		super(sg, I, Pi, false);
		assert sg instanceof TypeBasedStatic;
		assert I instanceof Individual [];
//		this.sg = sg;
//		this.Pi = Pi;
//		Ei = expectedUtilties();
//		E  = expectedExpectedUtility();
    }



	public String toString()
	{
		return this.multiline();
	}


	public String multiline()
	{
		StringBuffer s = new StringBuffer();

		for (int i = 0; i < Ei.length; i++)
		{
			if (i != 0) s.append('\n');
			s.append(Formatter.padr(i, 20));
			s.append(Formatter.padr(Ei[i], 20));
		}
		return s.toString();
	}


	/** Calculates the per-individu expected utility
	 *
	 * @return  Utilities
	 */

	public double [] calculateExpectedUtility(IIndividual [] I, ProbabilityDistribution Pi)
	{
		TypeBasedStatic sg = (TypeBasedStatic)S;

		double [] Ei = new double[sg.I.length];

		for (int i = 0; i < sg.I.length; i++)
		{
			// P(speaker=i)
			double P_si = Pi.ps[i]/2;
			if (DEBUG) System.out.println("P(speaker=" + i + ") = " + P_si);



			for (int j = 0; j < sg.I.length; j++)
			{
				// P(hearer=j)
				double P_hj = Pi.ps[j]/2;
				if (DEBUG) System.out.println("P(hearer=" + j + ") = " + P_hj);

				PayoffMatrix PU = sg.Utilities. matrices[I[i].reputationID][I[j].reputationID];
				PayoffMatrix PC = sg.Costs.     matrices[I[i].reputationID][I[j].reputationID];

				DiscreteFunction speakerStrategy = (DiscreteFunction)sg.I[i].Ss.fs[I[j].reputationID];
				DiscreteFunction hearerStrategy =  (DiscreteFunction)sg.I[j].Sh.fs[I[i].reputationID];

				for (int t = 0; t < sg.T.elements.length; t++)
				{
					// Determine what the speaker says in state t
					int m = speakerStrategy.mapping[t];

					// Determine what the hearere does when hearing that message
					int a = hearerStrategy.mapping[m];

					// P(state=t)
					double P_t = sg.Pt.ps[t];
//					if (DEBUG) System.out.println(Formatter.padr("P(speaker=" + i + ", hearer=" + j + ", state=" + t + ") = " + P_si_hj_t, 80) +
//													  Formatter.padr("Message utility = ", 30) +        Formatter.padl(sg.Costs.    values[t][m][0], 8) + "/ " + Formatter.padr(sg.Costs.    values[t][m][1], 8) +
//													  Formatter.padr("State-action utility = ", 30) +   Formatter.padl(sg.Utilities.values[t][a][0], 8) + "/ " + Formatter.padr(sg.Utilities.values[t][a][1], 8));



					// Determine the score this action and message give speaker and hearer in state t.
					Ei[i] += P_t * P_hj * (PU.values[t][a][0] + PC.values[t][m][0]);
					Ei[j] += P_t * P_si * (PU.values[t][a][1] + PC.values[t][m][1]);
				}
			}
		}
		return Ei;
	}


	/** Determine the next state in the evolution
	 *
	 * @return The next state
	 */

	public DynamicInformation evolve()
	{
		return new TypeBasedDynamic(S, I, S.mutator.mutate(S.dynamics.evolve(Ei, E, Pi)));
	}


}