package signalgame.typebased;

import java.util.*;
import signalgame.*;
import signalgame.typebased.*;
import math.*;

/** The static information of a Probability Based Signal Game.
 *  @author Project A.I. Development Team
 */

public class TypeBasedStatic extends StaticInformation
{
	public final	Individual []               I;
	public final    FiniteSet                   Is;
	public final    ProbabilityDistribution     Pi; // the P to start with!!!

	/** Constructs a TypeBased object, containing the static information during
	 *  the signal game simulation
	 *
	 * @param T         The set of States
	 * @param M         The set of messages
	 * @param A         The set of actions
	 * @param I         The set of individus
	 * @param Pt        The probability distribution over T
	 * @param Utilities The communication utility matrix
	 * @param Costs     The message utility matrix
	 * @param Es        The evolution step
	 */

	public TypeBasedStatic(  FiniteSet T, FiniteSet M, FiniteSet A, FiniteSet R,
						Individual [] I,
						ProbabilityDistribution Pt,
						ProbabilityDistribution Pi,
						ReputationPayoffMatrix Utilities, ReputationPayoffMatrix Costs,
						Mutator mutator)
	{
		super(T, M, A, R, Pt, Utilities, Costs, mutator);
		this.I =            I;
		this.Is =           new FiniteSet(I);
		this.Pi =           Pi;
	}


	public DynamicInformation createDyamic()
	{
		return new TypeBasedDynamic(this, I, Pi);
	}


	public String multiline()
	{
		StringBuffer s = new StringBuffer("Signal Game Configuration:");

		s.append("\nState Set:          T = " + T);
		s.append("\nProbabilities:      Pt= " + Pt);
		s.append("\nAction Set:         A = " + A);
		s.append("\nMessage Set:        M = " + M);
		s.append("\nReputation Set:     R = " + R);
//		s.append("\nIndividuals:        I = " + I.multiline());
		s.append("\nUtility Matrix:     UM = " + Utilities);
		s.append("\nCost Matrix:        CM = " + Costs);

		return s.toString();
	}
}