package test;

import math.*;
import util.*;
import signalgame.*;
import signalgame.instancebased.*;
import signalgame.typebased.*;
import java.util.Arrays;

/** After a while, under the right utility function and mutation, cooperation should occur.
 */


public class Cooperation
{
	public static final boolean DEBUG = false;

	public static void main(String [] args)
	{
		// Distinguish us from previous outputs
		System.out.println();

		// Create a simple 2x3 way system

		FiniteSet T = new FiniteSet(new Object [] {"s0", "s1"});
		FiniteSet M = new FiniteSet(new Object [] {"m0", "m1"});
		FiniteSet A = new FiniteSet(new Object [] {"a0", "a1"});
		FiniteSet R = new FiniteSet(new Object [] {"r0"});


		if (DEBUG) System.out.println("T: " + T);
		if (DEBUG) System.out.println("M: " + T);
		if (DEBUG) System.out.println("A: " + T);

		// With random state probability

		ProbabilityDistribution Pt = ProbabilityDistribution.random(T);

		if (DEBUG) System.out.println("Pt: " + Pt.multiline());

		// And some simple payoff tables

		double [][][] util =
			{{{1.0, 1.0}, {0.0, 0.0}},
			 {{0.0, 0.0}, {1.0, 1.0}}};

		double [][][] cost =
			{{{0.0, 0.0}, {0.0, 0.0}},
			 {{0.0, 0.0}, {0.0, 0.0}}};

		PayoffMatrix Utilities = new PayoffMatrix(T, A, util);
		PayoffMatrix Costs =     new PayoffMatrix(T, M, cost);

		ReputationPayoffMatrix UtilityR =   new ReputationPayoffMatrix(R, T, A, new double [][][][][] {{util}});
		ReputationPayoffMatrix CostR =      new ReputationPayoffMatrix(R, T, A, new double [][][][][] {{cost}});


		if (DEBUG) System.out.println("U: " + Utilities.multiline());
		if (DEBUG) System.out.println("C: " + Costs.multiline());


		// Now create a probability based set of individuals (just all possible)

		FiniteSet Ss =      DiscreteFunction.allPossible(T, M);
		FiniteSet Sh =      DiscreteFunction.allPossible(M, A);
		FiniteSet Is =      FiniteSet.cartesianProduct(Ss, Sh);
		signalgame.typebased.Individual [] I = signalgame.typebased.Individual.getIndividuals(R, Is);

		if (DEBUG) System.out.println("Ss: " + Ss);
		if (DEBUG) System.out.println("Sh: " + Sh);

		int individu = (int)(Math.random() * Is.elements.length);

		int POPULATIONSIZE = 100;

		// Create a probability distribution over all indivudials
		double []  ps =                 new double  [Is.elements.length];
		ps[individu] =                  1.0;
		ProbabilityDistribution Pi =    new ProbabilityDistribution(Is, ps);

		if (DEBUG) System.out.println(Pi.multiline());

		// Create the corresponding probability based individual set
		signalgame.typebased.Individual tbi = I[individu];
		signalgame.instancebased.Individual ibi = new signalgame.instancebased.Individual(null,
		    new ReputationFunction(R, new Function [] { new ProbabilisticFunction((DiscreteFunction)tbi.Ss.fs[0])}),
			new ReputationFunction(R, new Function [] { new ProbabilisticFunction((DiscreteFunction)tbi.Sh.fs[0])}),
			0);

		signalgame.instancebased.Individual [] Ipop = new signalgame.instancebased.Individual [POPULATIONSIZE];
		Arrays.fill(Ipop, ibi);

		// Now create instances

		Mutator mutator1 = new FairShareMutator(0.01);
		Mutator mutator2 = new ExchangeMutator();

		TypeBasedStatic s =         new TypeBasedStatic    (T, M, A, R, I,  Pt, Pi, UtilityR, CostR, mutator1);
		InstanceBasedStatic p =     new InstanceBasedStatic(T, M, A, R,     Pt,     UtilityR, CostR, mutator2);

		DynamicInformation si =     s.createDyamic();
		DynamicInformation pi =     p.createDyamic();

		int STEPS = 50;


		for (int i = 0; i < STEPS; i++)
		{
			si = (TypeBasedDynamic)si.evolve();
			pi = (InstanceBasedDynamic)pi.evolve();
		}

		System.out.println("Probability based:\n" + si.multiline());
		System.out.println("InstanceBased based:\n" + pi.multiline());

		System.out.println("Probability based best:\n" + si.getMostUtilous().multiline());
		System.out.println("InstanceBased based best:\n" + pi.getMostUtilous().multiline());
	}
}