package util;

/** Static Class to output controlled string information.
 *  @author Project A.I. Development Team
 */

public class Formatter
{
	/** Forwarding method.
	 * @see #padr(Object, int)
	 */
	public static String padr(int i,     int length) { return padr(Integer.toString(i),      length); }


	/** Forwarding method.
	 * @see #padr(Object, int)
	 */
	public static String padr(double d,  int length) { return padr(Double.toString(d),       length); }

	/** Forwarding method.
	 * @see #padr(Object, int)
	 */
	public static String padr(char c,    int length) { return padr(Character.toString(c),    length); }


	/** Forwarding method.
	 * @see #padl(Object, int)
	 */
	public static String padl(int i,     int length) { return padl(Integer.toString(i),      length); }

	/** Forwarding method.
	 * @see #padl(Object, int)
	 */
	public static String padl(double d,  int length) { return padl(Double.toString(d),       length); }


	/** Forwarding method.
	 * @see #padl(Object, int)
	 */
	public static String padl(char c,    int length) { return padl(Character.toString(c),    length); }

	/** Pads the string with spaces to the right to get the specified total length.
	 * @param o         The object to be represented as string
	 * @param length    The desired total length
	 * @return          The padded string
	 */

	public static String padr(Object o, int length)
	{
		StringBuffer sb = new StringBuffer();
		String s = o.toString();

		if (s.length() > length)
		{
			sb.append(s.substring(0, length-3));
			sb.append("...");
		}
		else
		{
			sb.append(s);
	    	for (int i = s.length(); i < length; i++) sb.append(' ');
		}

		return sb.toString();
	}

	/** Pads the string with spaces to the left to get the specified total length.
	 * @param o         The object to be represented as string
	 * @param length    The desired total length
	 * @return          The padded string
	 */

	public static String padl(Object o, int length)
	{
		StringBuffer sb = new StringBuffer();
		String s = o.toString();

		if (s.length() > length)
		{
			sb.append(s.substring(0, length-3));
			sb.append("...");
		}
		else
		{
			for (int i = s.length(); i < length; i++) sb.append(' ');
			sb.append(s);
		}

		return sb.toString();
	}

	/** Replaces all unprintable characters with printable character sequences.
	 *  \t, \n and \r:                      replaced by their textual representation
	 *  unicode < 32 || 127 < unicode:      replaced by '#' plus thier unicode number
	 *
	 * @param o         The object to be unformatted
	 * @return          The unformatted string
	 */


	public static String unformat(Object o)
	{
		StringBuffer sb = new StringBuffer();
		String s = o.toString();
		int length = s.length();

		for (int i = 0; i < length; i++)
		{
			char c = s.charAt(i);
			     if (c == '\t') sb.append("\\t");
			else if (c == '\n') sb.append("\\n");
			else if (c == '\r') sb.append("\\r");
		    else if ((int)c < 32)  sb.append("(#" + (int)c + ")");
			else if ((int)c > 127) sb.append("(#" + (int)c + ")");
			else sb.append(c);
		}

		return sb.toString();
	}
}
